@@ -1,3 +1,4 @@
+use crate::backend::VersionInfo;
 use crate::backend::asset_detector;
 use crate::backend::backend_type::BackendType;
 use crate::backend::platform_target::PlatformTarget;
@@ -5,7 +6,7 @@ use crate::backend::static_helpers::{
     get_filename_from_url, install_artifact, lookup_platform_key, lookup_platform_key_for_target,
     template_string, try_with_v_prefix, verify_artifact,
 };
-use crate::cli::args::BackendArg;
+use crate::cli::args::{BackendArg, ToolVersionType};
 use crate::config::Config;
 use crate::file;
 use crate::http::HTTP;
@@ -36,6 +37,15 @@ struct ReleaseAsset {
 const DEFAULT_GITHUB_API_BASE_URL: &str = "https://api.github.com";
 const DEFAULT_GITLAB_API_BASE_URL: &str = "https://gitlab.com/api/v4";
 
+/// Returns install-time-only option keys for GitHub/GitLab backend.
+pub fn install_time_option_keys() -> Vec<String> {
+    vec![
+        "asset_pattern".into(),
+        "url".into(),
+        "version_prefix".into(),
+    ]
+}
+
 #[async_trait]
 impl Backend for UnifiedGitBackend {
     fn get_type(&self) -> BackendType {
@@ -50,34 +60,72 @@ impl Backend for UnifiedGitBackend {
         &self.ba
     }
 
-    async fn _list_remote_versions(&self, _config: &Arc<Config>) -> Result<Vec<String>> {
+    async fn _list_remote_versions_with_info(
+        &self,
+        _config: &Arc<Config>,
+    ) -> Result<Vec<VersionInfo>> {
         let repo = self.ba.tool_name();
+        let id = self.ba.to_string();
         let opts = self.ba.opts();
         let api_url = self.get_api_url(&opts);
         let version_prefix = opts.get("version_prefix");
 
-        // Get tag names from either GitHub or GitLab
-        let tag_names: Vec<String> = if self.is_gitlab() {
+        // Derive web URL base from API URL for enterprise support
+        let web_url_base = if self.is_gitlab() {
+            if api_url == DEFAULT_GITLAB_API_BASE_URL {
+                format!("https://gitlab.com/{}", repo)
+            } else {
+                // Enterprise GitLab - derive web URL from API URL
+                let web_url = api_url.replace("/api/v4", "");
+                format!("{}/{}", web_url, repo)
+            }
+        } else if api_url == DEFAULT_GITHUB_API_BASE_URL {
+            format!("https://github.com/{}", repo)
+        } else {
+            // Enterprise GitHub - derive web URL from API URL
+            let web_url = api_url.replace("/api/v3", "").replace("api.", "");
+            format!("{}/{}", web_url, repo)
+        };
+
+        // Get releases with full metadata from GitHub or GitLab
+        let raw_versions: Vec<VersionInfo> = if self.is_gitlab() {
             gitlab::list_releases_from_url(api_url.as_str(), &repo)
                 .await?
                 .into_iter()
-                .map(|r| r.tag_name)
+                .filter(|r| version_prefix.is_none_or(|p| r.tag_name.starts_with(p)))
+                .map(|r| VersionInfo {
+                    version: self.strip_version_prefix(&r.tag_name),
+                    created_at: r.released_at,
+                    release_url: Some(format!("{}/-/releases/{}", web_url_base, r.tag_name)),
+                })
                 .collect()
         } else {
             github::list_releases_from_url(api_url.as_str(), &repo)
                 .await?
                 .into_iter()
-                .map(|r| r.tag_name)
+                .filter(|r| version_prefix.is_none_or(|p| r.tag_name.starts_with(p)))
+                .map(|r| VersionInfo {
+                    version: self.strip_version_prefix(&r.tag_name),
+                    created_at: Some(r.created_at),
+                    release_url: Some(format!("{}/releases/tag/{}", web_url_base, r.tag_name)),
+                })
                 .collect()
         };
 
-        // Apply common filtering and mapping
-        Ok(tag_names
+        // Apply common validation and reverse order
+        let versions = raw_versions
             .into_iter()
-            .filter(|tag| version_prefix.is_none_or(|p| tag.starts_with(p)))
-            .map(|tag| self.strip_version_prefix(&tag))
+            .filter(|v| match v.version.parse::<ToolVersionType>() {
+                Ok(ToolVersionType::Version(_)) => true,
+                _ => {
+                    warn!("Invalid version: {id}@{}", v.version);
+                    false
+                }
+            })
             .rev()
-            .collect())
+            .collect();
+
+        Ok(versions)
     }
 
     async fn install_version_(
