@@ -21,6 +21,9 @@ use strum::{AsRefStr, EnumString, VariantNames};
 use url::Url;
 use xx::regex;
 
+/// SPM backend requires experimental mode to be enabled
+pub const EXPERIMENTAL: bool = true;
+
 #[derive(Debug)]
 pub struct SPMBackend {
     ba: Arc<BackendArg>,
@@ -139,6 +142,9 @@ impl SPMBackend {
         debug!("Checking out revision: {revision}");
         repo.update_tag(revision.to_string())?;
 
+        // Updates submodules ensuring they match the checked-out revision
+        repo.update_submodules()?;
+
         Ok(repo.dir)
     }
 
