@@ -163,11 +163,14 @@ impl Lock {
         let mut platforms: BTreeSet<Platform> = Platform::common_platforms().into_iter().collect();
         platforms.insert(Platform::current());
 
-        // Add any existing platforms from lockfile
+        // Add any existing platforms from lockfile (only valid ones)
         if let Ok(lockfile) = Lockfile::read(lockfile_path) {
             for platform_key in lockfile.all_platform_keys() {
                 if let Ok(p) = Platform::parse(&platform_key) {
-                    platforms.insert(p);
+                    // Skip invalid platforms (e.g., tool-specific qualifiers like "wait-for-gh-rate-limit")
+                    if p.validate().is_ok() {
+                        platforms.insert(p);
+                    }
                 }
             }
         }
@@ -180,21 +183,39 @@ impl Lock {
         config: &Config,
         ts: &Toolset,
     ) -> Vec<(crate::cli::args::BackendArg, crate::toolset::ToolVersion)> {
-        // Collect tools from ALL config files (not just resolved current versions)
-        // This ensures base config tools are included even when overridden by env configs
+        // Calculate target config_root (same logic as get_lockfile_path)
+        let target_root = config
+            .config_files
+            .keys()
+            .next()
+            .map(|p| config_root::config_root(p))
+            .unwrap_or_else(|| std::env::current_dir().unwrap_or_default());
+
+        // Collect tools from config files in the target config_root only
         let mut all_tools: Vec<_> = Vec::new();
         let mut seen: BTreeSet<(String, String)> = BTreeSet::new();
 
         // First, get all tools from the resolved toolset (these are the "current" versions)
+        // but only if they come from a config file in the target config_root
         for (backend, tv) in ts.list_current_versions() {
+            // Check if this tool's source is in the target config_root
+            if let Some(source_path) = tv.request.source().path()
+                && config_root::config_root(source_path) != target_root
+            {
+                continue;
+            }
             let key = (backend.ba().short.clone(), tv.version.clone());
             if seen.insert(key) {
                 all_tools.push((backend.ba().as_ref().clone(), tv));
             }
         }
 
-        // Then, iterate ALL config files to find tools that may have been overridden
-        for (_path, cf) in config.config_files.iter() {
+        // Then, iterate config files in the target config_root to find tools that may have been overridden
+        for (path, cf) in config.config_files.iter() {
+            // Skip config files not in the target config_root
+            if config_root::config_root(path) != target_root {
+                continue;
+            }
             if let Ok(trs) = cf.to_tool_request_set() {
                 for (ba, requests, _source) in trs.iter() {
                     for request in requests {
@@ -211,10 +232,9 @@ impl Lock {
                                     }
                                 }
                             }
-                            // Also check installed versions that match this request
-                            let installed = backend.list_installed_versions();
+                            // For "latest" requests, find the highest installed version
                             if request.version() == "latest" {
-                                // For "latest", find the highest installed version
+                                let installed = backend.list_installed_versions();
                                 if let Some(latest_version) = installed.iter().max_by(|a, b| {
                                     versions::Versioning::new(a).cmp(&versions::Versioning::new(b))
                                 }) {
@@ -227,21 +247,6 @@ impl Lock {
                                         all_tools.push((ba.as_ref().clone(), tv));
                                     }
                                 }
-                            } else {
-                                // For prefix requests, find matching versions using proper fuzzy matching
-                                // (ensures "1" matches "1.0.0" but not "10.0.0")
-                                for version in
-                                    backend.list_installed_versions_matching(&request.version())
-                                {
-                                    let key = (ba.short.clone(), version.clone());
-                                    if seen.insert(key.clone()) {
-                                        let tv = crate::toolset::ToolVersion::new(
-                                            request.clone(),
-                                            version,
-                                        );
-                                        all_tools.push((ba.as_ref().clone(), tv));
-                                    }
-                                }
                             }
                         }
                     }
@@ -270,14 +275,23 @@ impl Lock {
     ) -> Result<()> {
         miseprintln!("{} Dry run - would update:", style("→").yellow());
         for (ba, tv) in tools {
+            let backend = crate::backend::get(ba);
             for platform in platforms {
-                miseprintln!(
-                    "  {} {}@{} for {}",
-                    style("✓").green(),
-                    style(&ba.short).bold(),
-                    tv.version,
-                    style(platform.to_key()).blue()
-                );
+                // Expand platform variants just like process_tools does
+                let variants = if let Some(ref backend) = backend {
+                    backend.platform_variants(platform)
+                } else {
+                    vec![platform.clone()]
+                };
+                for variant in variants {
+                    miseprintln!(
+                        "  {} {}@{} for {}",
+                        style("✓").green(),
+                        style(&ba.short).bold(),
+                        tv.version,
+                        style(variant.to_key()).blue()
+                    );
+                }
             }
         }
         Ok(())
@@ -296,56 +310,69 @@ impl Lock {
         let mut results = Vec::new();
 
         let mpr = MultiProgressReport::get();
-        let total_tasks = tools.len() * platforms.len();
+
+        // Collect all platform variants for each tool/platform combination
+        let mut all_tasks: Vec<(
+            crate::cli::args::BackendArg,
+            crate::toolset::ToolVersion,
+            Platform,
+        )> = Vec::new();
+        for (ba, tv) in tools {
+            let backend = crate::backend::get(ba);
+            for platform in platforms {
+                // Get all variants for this platform from the backend
+                let variants = if let Some(ref backend) = backend {
+                    backend.platform_variants(platform)
+                } else {
+                    vec![platform.clone()]
+                };
+                for variant in variants {
+                    all_tasks.push((ba.clone(), tv.clone(), variant));
+                }
+            }
+        }
+
+        let total_tasks = all_tasks.len();
         let pr = mpr.add("lock");
         pr.set_length(total_tasks as u64);
 
-        // Spawn tasks for each tool/platform combination
-        for (ba, tv) in tools {
-            for platform in platforms {
-                let ba = ba.clone();
-                let tv = tv.clone();
-                let platform = platform.clone();
-                let semaphore = semaphore.clone();
+        // Spawn tasks for each tool/platform variant combination
+        for (ba, tv, platform) in all_tasks {
+            let semaphore = semaphore.clone();
 
-                jset.spawn(async move {
-                    let _permit = semaphore.acquire().await;
-                    let target = PlatformTarget::new(platform.clone());
-                    let backend = crate::backend::get(&ba);
+            jset.spawn(async move {
+                let _permit = semaphore.acquire().await;
+                let target = PlatformTarget::new(platform.clone());
+                let backend = crate::backend::get(&ba);
 
-                    let (info, options) = if let Some(backend) = backend {
-                        let options = backend.resolve_lockfile_options(&tv.request, &target);
-                        match backend.resolve_lock_info(&tv, &target).await {
-                            Ok(info) if info.url.is_some() => (Some(info), options),
-                            Ok(_) => {
-                                debug!("No URL found for {} on {}", ba.short, platform.to_key());
-                                (None, options)
-                            }
-                            Err(e) => {
-                                warn!(
-                                    "Failed to resolve {} for {}: {}",
-                                    ba.short,
-                                    platform.to_key(),
-                                    e
-                                );
-                                (None, options)
-                            }
+                let (info, options) = if let Some(backend) = backend {
+                    let options = backend.resolve_lockfile_options(&tv.request, &target);
+                    match backend.resolve_lock_info(&tv, &target).await {
+                        Ok(info) => (Some(info), options),
+                        Err(e) => {
+                            warn!(
+                                "Failed to resolve {} for {}: {}",
+                                ba.short,
+                                platform.to_key(),
+                                e
+                            );
+                            (None, options)
                         }
-                    } else {
-                        warn!("Backend not found for {}", ba.short);
-                        (None, BTreeMap::new())
-                    };
+                    }
+                } else {
+                    warn!("Backend not found for {}", ba.short);
+                    (None, BTreeMap::new())
+                };
 
-                    (
-                        ba.short.clone(),
-                        tv.version.clone(),
-                        ba.full(),
-                        platform,
-                        info,
-                        options,
-                    )
-                });
-            }
+                (
+                    ba.short.clone(),
+                    tv.version.clone(),
+                    ba.full(),
+                    platform,
+                    info,
+                    options,
+                )
+            });
         }
 
         // Collect all results
