@@ -22,15 +22,17 @@ use crate::config::config_file::{config_root, toml::deserialize_arr};
 use crate::config::env_directive::{AgeFormat, EnvDirective, EnvDirectiveOptions, RequiredValue};
 use crate::config::settings::SettingsPartial;
 use crate::config::{Alias, AliasMap, Config};
-use crate::file;
+use crate::env_diff::EnvMap;
 use crate::file::{create_dir_all, display_path};
 use crate::hooks::{Hook, Hooks};
+use crate::prepare::PrepareConfig;
 use crate::redactions::Redactions;
 use crate::registry::REGISTRY;
 use crate::task::Task;
 use crate::tera::{BASE_CONTEXT, get_tera};
 use crate::toolset::{ToolRequest, ToolRequestSet, ToolSource, ToolVersionOptions};
 use crate::watch_files::WatchFile;
+use crate::{env, file};
 
 use super::{ConfigFileType, min_version::MinVersionSpec};
 
@@ -52,6 +54,10 @@ pub struct MiseToml {
     env_path: Vec<String>,
     #[serde(default)]
     alias: AliasMap,
+    #[serde(default)]
+    tool_alias: AliasMap,
+    #[serde(default)]
+    shell_alias: IndexMap<String, String>,
     #[serde(skip)]
     doc: Mutex<OnceCell<DocumentMut>>,
     #[serde(default)]
@@ -69,6 +75,8 @@ pub struct MiseToml {
     #[serde(default)]
     watch_files: Vec<WatchFile>,
     #[serde(default)]
+    prepare: Option<PrepareConfig>,
+    #[serde(default)]
     vars: EnvList,
     #[serde(default)]
     settings: SettingsPartial,
@@ -137,11 +145,13 @@ impl MiseToml {
             "config_root",
             config_root::config_root(path).to_str().unwrap(),
         );
-        Self {
+        let mut rf = Self {
             path: path.to_path_buf(),
             context,
             ..Default::default()
-        }
+        };
+        rf.update_context_env(env::PRISTINE_ENV.clone());
+        rf
     }
 
     pub fn from_file(path: &Path) -> eyre::Result<Self> {
@@ -164,8 +174,11 @@ impl MiseToml {
             }
         };
         rf.context = BASE_CONTEXT.clone();
-        rf.context
-            .insert("config_root", path.parent().unwrap().to_str().unwrap());
+        rf.context.insert(
+            "config_root",
+            config_root::config_root(path).to_str().unwrap(),
+        );
+        rf.update_context_env(env::PRISTINE_ENV.clone());
         rf.path = path.to_path_buf();
         let project_root = rf.project_root().map(|p| p.to_path_buf());
         for task in rf.tasks.0.values_mut() {
@@ -356,6 +369,23 @@ impl MiseToml {
         Ok(())
     }
 
+    // Merge base OS env vars with env sections from this file,
+    // so they are available for templating.
+    // Note this only merges regular key-value variables; referenced files are not resolved.
+    fn update_context_env(&mut self, mut base_env: EnvMap) {
+        let env_vars = self
+            .env
+            .0
+            .iter()
+            .filter_map(|e| match e {
+                EnvDirective::Val(key, value, _) => Some((key.clone(), value.clone())),
+                _ => None,
+            })
+            .collect::<IndexMap<_, _>>();
+        base_env.extend(env_vars);
+        self.context.insert("env", &base_env);
+    }
+
     fn parse_template(&self, input: &str) -> eyre::Result<String> {
         self.parse_template_with_context(&self.context, input)
     }
@@ -581,10 +611,29 @@ impl ConfigFile for MiseToml {
                         *v = self.parse_template_with_context(&context, v)?;
                     }
                     let mut ba = ba.clone();
+                    // Start with cached options but filter out install-time-only options
+                    // when config provides its own options. This allows:
+                    // - Changing url/asset_pattern/checksum without reinstall issues
+                    // - Preserving post-install options like bin_path for binary discovery
                     let mut ba_opts = ba.opts().clone();
+                    let install_time_keys =
+                        crate::backend::install_time_option_keys_for_type(&ba.backend_type());
+                    if !install_time_keys.is_empty() {
+                        ba_opts.opts.retain(|k, _| {
+                            // Keep option if it's NOT an install-time-only key
+                            // Also filter platform-specific variants (platforms.X.key)
+                            !install_time_keys.contains(k)
+                                && !install_time_keys.iter().any(|itk| {
+                                    k.starts_with("platforms.") && k.ends_with(&format!(".{itk}"))
+                                })
+                        });
+                    }
                     ba_opts.merge(&options.opts);
+                    // Copy os and install_env from config (not cached)
+                    ba_opts.os = options.os.clone();
+                    ba_opts.install_env = options.install_env.clone();
                     ba.set_opts(Some(ba_opts.clone()));
-                    ToolRequest::new_opts(ba.into(), &version, options, source.clone())?
+                    ToolRequest::new_opts(ba.into(), &version, ba_opts, source.clone())?
                 } else {
                     ToolRequest::new(ba.clone().into(), &version, source.clone())?
                 };
@@ -595,8 +644,22 @@ impl ConfigFile for MiseToml {
     }
 
     fn aliases(&self) -> eyre::Result<AliasMap> {
-        self.alias
-            .clone()
+        // Emit deprecation warning if [alias] is used
+        if !self.alias.is_empty() {
+            deprecated!(
+                "alias",
+                "[alias] is deprecated, use [tool_alias] instead in {}",
+                display_path(&self.path)
+            );
+        }
+
+        // Merge alias and tool_alias, with tool_alias taking precedence
+        let mut combined: AliasMap = self.alias.clone();
+        for (k, v) in &self.tool_alias {
+            combined.insert(k.clone(), v.clone());
+        }
+
+        combined
             .iter()
             .map(|(k, v)| {
                 let versions = v
@@ -619,6 +682,16 @@ impl ConfigFile for MiseToml {
             .collect()
     }
 
+    fn shell_aliases(&self) -> eyre::Result<IndexMap<String, String>> {
+        self.shell_alias
+            .iter()
+            .map(|(k, v)| {
+                let v = self.parse_template(v)?;
+                Ok((k.clone(), v))
+            })
+            .collect()
+    }
+
     fn task_config(&self) -> &TaskConfig {
         &self.task_config
     }
@@ -666,6 +739,10 @@ impl ConfigFile for MiseToml {
             .flatten()
             .collect())
     }
+
+    fn prepare_config(&self) -> Option<PrepareConfig> {
+        self.prepare.clone()
+    }
 }
 
 /// Returns a [`toml_edit::Key`] from the given `key`.
@@ -723,6 +800,8 @@ impl Clone for MiseToml {
             env: self.env.clone(),
             env_path: self.env_path.clone(),
             alias: self.alias.clone(),
+            tool_alias: self.tool_alias.clone(),
+            shell_alias: self.shell_alias.clone(),
             doc: Mutex::new(self.doc.lock().unwrap().clone()),
             hooks: self.hooks.clone(),
             tools: Mutex::new(self.tools.lock().unwrap().clone()),
@@ -732,6 +811,7 @@ impl Clone for MiseToml {
             task_config: self.task_config.clone(),
             settings: self.settings.clone(),
             watch_files: self.watch_files.clone(),
+            prepare: self.prepare.clone(),
             vars: self.vars.clone(),
             experimental_monorepo_root: self.experimental_monorepo_root,
         }
@@ -1731,6 +1811,30 @@ mod tests {
         });
     }
 
+    #[tokio::test]
+    async fn test_env_var_in_tool() {
+        let _config = Config::get().await.unwrap();
+        let p = CWD.as_ref().unwrap().join(".test.mise.toml");
+        file::write(
+            &p,
+            r#"
+        [env]
+        TERRAFORM_VERSION = '1.0.0'
+        JQ_PREFIX = '1.6'
+
+        [tools]
+        terraform = "{{env.TERRAFORM_VERSION}}"
+        jq = { prefix = "{{ env.JQ_PREFIX }}" }
+        "#,
+        )
+        .unwrap();
+        let cf = MiseToml::from_file(&p).unwrap();
+        assert_snapshot!(replace_path(&format!(
+            "{:#?}",
+            cf.to_tool_request_set().unwrap().tools
+        )));
+    }
+
     #[tokio::test]
     async fn test_env_array_valid() {
         let _config = Config::get().await.unwrap();
