@@ -4,7 +4,7 @@ use std::fs::{self};
 use std::path::{Path, PathBuf};
 use std::sync::Arc;
 
-use crate::backend::Backend;
+use crate::backend::{Backend, VersionInfo};
 use crate::cache::{CacheManager, CacheManagerBuilder};
 use crate::cli::args::BackendArg;
 use crate::cli::version::OS;
@@ -301,6 +301,18 @@ impl Backend for JavaPlugin {
     }
 
     async fn _list_remote_versions(&self, config: &Arc<Config>) -> Result<Vec<String>> {
+        Ok(self
+            ._list_remote_versions_with_info(config)
+            .await?
+            .into_iter()
+            .map(|v| v.version)
+            .collect())
+    }
+
+    async fn _list_remote_versions_with_info(
+        &self,
+        config: &Arc<Config>,
+    ) -> Result<Vec<VersionInfo>> {
         let release_type = config
             .get_tool_request_set()
             .await?
@@ -348,8 +360,12 @@ impl Backend for JavaPlugin {
                     v.to_string(),
                 )
             })
-            .map(|(v, _)| v.clone())
-            .unique()
+            .map(|(v, m)| VersionInfo {
+                version: v.clone(),
+                created_at: m.created_at.clone(),
+                ..Default::default()
+            })
+            .unique_by(|v| v.version.clone())
             .collect();
 
         Ok(versions)
@@ -482,13 +498,21 @@ impl Backend for JavaPlugin {
     }
 
     fn fuzzy_match_filter(&self, versions: Vec<String>, query: &str) -> Vec<String> {
+        let is_vendor_prefix = query != "latest" && query.ends_with('-');
         let query_escaped = regex::escape(query);
         let query = match query {
             "latest" => "[0-9].*",
             // else; use escaped query
             _ => &query_escaped,
         };
-        let query_regex = Regex::new(&format!("^{query}([+-.].+)?$")).unwrap();
+        // Same semantics as Backend::fuzzy_match_filter:
+        // - "1.2" should match "1.2.3" but not "1.20"
+        // - vendor prefixes like "temurin-" should match "temurin-25..."
+        let query_regex = if is_vendor_prefix {
+            Regex::new(&format!("^{query}.*$")).unwrap()
+        } else {
+            Regex::new(&format!("^{query}([+\\-.].+)?$")).unwrap()
+        };
 
         versions
             .into_iter()
@@ -530,6 +554,7 @@ struct JavaMetadata {
     // architecture: String,
     checksum: Option<String>,
     // checksum_url: Option<String>,
+    created_at: Option<String>,
     features: Option<Vec<String>>,
     file_type: Option<String>,
     // filename: String,
