@@ -5,6 +5,7 @@ use std::{
 };
 
 use crate::backend::Backend;
+use crate::backend::VersionInfo;
 use crate::backend::platform_target::PlatformTarget;
 use crate::cli::args::BackendArg;
 use crate::cmd::CmdLineRunner;
@@ -13,6 +14,7 @@ use crate::duration::DAILY;
 use crate::file::TarOptions;
 use crate::http::{HTTP, HTTP_FETCH};
 use crate::install_context::InstallContext;
+use crate::lockfile::PlatformInfo;
 use crate::toolset::ToolVersion;
 use crate::ui::progress_report::SingleReport;
 use crate::{file, minisign, plugins};
@@ -172,6 +174,39 @@ impl ZigPlugin {
         Ok(zig_tarball_url.to_string())
     }
 
+    /// Get full download info (tarball URL, shasum, size) from JSON index
+    /// Uses cached request since same index is fetched for all platforms
+    async fn get_download_info_from_json(
+        &self,
+        json_url: &str,
+        version: &str,
+        arch: &str,
+        os: &str,
+    ) -> Result<(String, Option<String>, Option<u64>)> {
+        let version_json: serde_json::Value = HTTP_FETCH.json_cached(json_url).await?;
+        let platform_info = version_json
+            .pointer(&format!("/{version}/{arch}-{os}"))
+            .ok_or_else(|| eyre::eyre!("Failed to get zig platform info from {:?}", json_url))?;
+
+        let tarball_url = platform_info
+            .get("tarball")
+            .and_then(|v| v.as_str())
+            .ok_or_else(|| eyre::eyre!("Failed to get zig tarball url from {:?}", json_url))?
+            .to_string();
+
+        let shasum = platform_info
+            .get("shasum")
+            .and_then(|v| v.as_str())
+            .map(|s| format!("sha256:{s}"));
+
+        let size = platform_info
+            .get("size")
+            .and_then(|v| v.as_str())
+            .and_then(|s| s.parse::<u64>().ok());
+
+        Ok((tarball_url, shasum, size))
+    }
+
     async fn get_community_mirrors(&self) -> Option<Vec<String>> {
         let cache_path = self.ba.cache_path.join(MIRRORS_FILENAME);
         let recent_cache =
@@ -207,29 +242,50 @@ impl Backend for ZigPlugin {
         &self.ba
     }
 
-    async fn _list_remote_versions(&self, _config: &Arc<Config>) -> Result<Vec<String>> {
+    async fn security_info(&self) -> Vec<crate::backend::SecurityFeature> {
+        use crate::backend::SecurityFeature;
+
+        vec![
+            SecurityFeature::Checksum {
+                algorithm: Some("sha256".to_string()),
+            },
+            SecurityFeature::Minisign {
+                public_key: Some(ZIG_MINISIGN_KEY.to_string()),
+            },
+        ]
+    }
+
+    async fn _list_remote_versions_with_info(
+        &self,
+        _config: &Arc<Config>,
+    ) -> Result<Vec<VersionInfo>> {
         let indexes = [
             "https://ziglang.org/download/index.json",
             // "https://machengine.org/zig/index.json", // need to handle mach's CalVer
         ];
-        let mut versions: Vec<String> = Vec::new();
+        let mut versions: Vec<(String, Option<String>)> = Vec::new();
 
         for index in indexes {
             let index_json: serde_json::Value = HTTP_FETCH.json(index).await?;
-            let index_versions: Vec<String> = index_json
+            let index_obj = index_json
                 .as_object()
-                .ok_or_else(|| eyre::eyre!("Failed to get zig version from {:?}", index))?
-                .keys()
-                .cloned()
-                .collect();
+                .ok_or_else(|| eyre::eyre!("Failed to get zig version from {:?}", index))?;
 
-            versions.extend(index_versions);
+            for (version, data) in index_obj {
+                let date = data.get("date").and_then(|d| d.as_str()).map(String::from);
+                versions.push((version.clone(), date));
+            }
         }
 
         let versions = versions
             .into_iter()
-            .unique()
-            .sorted_by_cached_key(|s| (Versioning::new(s), s.to_string()))
+            .unique_by(|(v, _)| v.clone())
+            .sorted_by_cached_key(|(s, _)| (Versioning::new(s), s.to_string()))
+            .map(|(version, date)| VersionInfo {
+                version,
+                created_at: date,
+                ..Default::default()
+            })
             .collect();
 
         Ok(versions)
@@ -309,4 +365,62 @@ impl Backend for ZigPlugin {
             Err(_) => Ok(None),
         }
     }
+
+    async fn resolve_lock_info(
+        &self,
+        tv: &ToolVersion,
+        target: &PlatformTarget,
+    ) -> Result<PlatformInfo> {
+        let indexes = HashMap::from([
+            ("zig", "https://ziglang.org/download/index.json"),
+            ("mach", "https://machengine.org/zig/index.json"),
+        ]);
+
+        let arch = match target.arch_name() {
+            "x64" => "x86_64",
+            "arm64" => "aarch64",
+            "arm" => "armv7a",
+            "riscv64" => "riscv64",
+            other => other,
+        };
+        let os = match target.os_name() {
+            "macos" => "macos",
+            "linux" => "linux",
+            "freebsd" => "freebsd",
+            "windows" => "windows",
+            _ => "linux",
+        };
+
+        let (json_url, version) = if regex!(r"^mach-|-mach$").is_match(&tv.version) {
+            (indexes["mach"], tv.version.as_str())
+        } else {
+            (indexes["zig"], tv.version.as_str())
+        };
+
+        // Try to get full info from JSON (includes checksum and size)
+        match self
+            .get_download_info_from_json(json_url, version, arch, os)
+            .await
+        {
+            Ok((url, checksum, size)) => Ok(PlatformInfo {
+                url: Some(url),
+                checksum,
+                size,
+                url_api: None,
+            }),
+            Err(_) if regex!(r"^\d+\.\d+\.\d+$").is_match(&tv.version) => {
+                // Fallback: construct URL directly for numbered versions (no checksum available)
+                Ok(PlatformInfo {
+                    url: Some(format!(
+                        "https://ziglang.org/download/{}/zig-{}-{}-{}.tar.xz",
+                        tv.version, os, arch, tv.version
+                    )),
+                    checksum: None,
+                    size: None,
+                    url_api: None,
+                })
+            }
+            Err(_) => Ok(PlatformInfo::default()),
+        }
+    }
 }
