@@ -16,14 +16,9 @@ pub fn init() {
                 let _ = Term::stderr().show_cursor();
             }
             CmdLineRunner::kill_all(nix::sys::signal::SIGINT);
-            if EXIT.swap(true, Ordering::Relaxed) {
+            if EXIT.load(Ordering::Relaxed) {
                 debug!("Ctrl-C pressed, exiting...");
                 exit(1);
-            } else {
-                eprintln!();
-                warn!(
-                    "Ctrl-C pressed, please wait for tasks to finish or press Ctrl-C again to force exit"
-                );
             }
         }
     });
